//
//  View.swift
//  Puzzle
//
//  Created by Mark Meretzky on 11/15/14.
//  Copyright (c) 2014 Mark Meretzky. All rights reserved.
//

import UIKit

class View: UIView {
	var emptyRow: Int = 1;	//Lower right position is initially empty.
	var emptyCol: Int = 1;

	override init(frame: CGRect) {
		super.init(frame: frame);
		self.backgroundColor = UIColor.whiteColor();

		for var row = -1; row <= 1; ++row {
			for var col = -1; col <= 1; ++col {

				//Skip the missing tile.
				if row == emptyRow && col == emptyCol {
					continue;
				}

				let tileView: TileView = TileView(row: row, col: col);
				addSubview(tileView);
			}
		}

		//Let the center of this View be the origin (0, 0).
		bounds.origin = CGPointMake(
			-bounds.size.width / 2,
			-bounds.size.height / 2);
	}

	//Never called.
	required init(coder aDecoder: NSCoder) {
		super.init(coder: aDecoder);
	}

	override func touchesBegan(touches: NSSet, withEvent event: UIEvent) {
		let touch: UITouch = touches.anyObject() as UITouch;
		let point: CGPoint = touch.locationInView(self);

		//Which of the 8 subviews (TileViews) was touched?
		let view: UIView? = hitTest(point, withEvent: event);
		if view == nil || view == self {
			return;	//No TileView was touched.
		}

		let tileView: TileView = view as TileView;
		tryToMoveTileView(tileView);
	}

	//Move the given TileView if it is not landlocked.

	func tryToMoveTileView(tileView: TileView) {
		//Can the TileView move into the empty location?
		//Or is the TileView landlocked?
		let dRow: Int = abs(tileView.row - emptyRow);
		let dCol: Int = abs(tileView.col - emptyCol);
		if dRow + dCol != 1 {
			return;	//Landlocked.
		}

		//Remember where the TileView is.
		let saveRow: Int = tileView.row;
		let saveCol: Int = tileView.col;

		//Move the TileView into the empty location.
		UIView.animateWithDuration(0.2,
			delay: 0,
			options: UIViewAnimationOptions.CurveEaseInOut,
			animations: {tileView.moveToRow(self.emptyRow, col: self.emptyCol);},
			completion: nil);

		//The location where the TileView was is now the empty location.
		emptyRow = saveRow;
		emptyCol = saveCol;
	}

	/*
	// Only override drawRect: if you perform custom drawing.
	// An empty implementation adversely affects performance during animation.
	override func drawRect(rect: CGRect) {
		// Drawing code
	}
	*/

}
